# LSB Test Execution Framework
# Common functionality module
#
# Copyright (C) 2006 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2006 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Common;
use strict;

use MIME::Base64 qw(encode_base64);

# Export symbols
require Exporter;
@Common::ISA = qw(Exporter);
push @Common::EXPORT, qw(
    $tester_email $send_email $verbose_lvl $lsb_version
    $host_machine $host_platform $host_name $host_os $host_kernel
    $host_triplet $host_etc_issue $host_package_manager $wget 
    $script_directory $download_packages $package_dir 
    $try_beta_packages $keep_packages $output_file_name 
    $start_time $finish_time 
    @tests_to_run %tests %test_mode 
    $test_comments $results_dir 
    $omit_packing $omit_INFO
    $short_scenarios $autotest_cmd
    $list_refresh $autotest_dir
    $certification_mode
);

push @Common::EXPORT, qw(
    &print_ref
    &complain &fail_nomail &fail &inform &run_command &test_command 
    &write_string_as_file &does_shell_know &check_for_tool &debug_inform
    &create_empty_directory &extract_filename &cmd &read_file &sendmail
    &backup_and_replace &capture
);

#==== Global variables ==================================================

# verbose level: -v[0-2]
#   0 - minimal. Only error messages are put into stderr; 
#   1 - default; 
#   2 - extended debug information.
our $verbose_lvl;

# Properties of the host machine.
our $host_machine;
our $host_platform;
our $host_name;
our $host_os;
our $host_kernel;
our $host_triplet;
our $host_etc_issue;
our $host_package_manager;
    
# Components release version
our $lsb_version;

# How the wget tool should be called.
our $wget;

# The directory where the script itself is placed.
our $script_directory;

# The directory where autotest is placed
our $autotest_dir;

# Command line parameters
our $download_packages;
our $package_dir;
our $output_file_name;
our $try_beta_packages;
our $keep_packages;
our $omit_packing;
our $omit_INFO;
our $short_scenarios;
our $list_refresh;
our $certification_mode;
our $tester_email;
our $send_email;

# Start and finish time
our $start_time;
our $finish_time;

# Array of tests to run; they will be run in the specified order. 
our @tests_to_run;

# Tests start settings: which way each test shall be installed
our %test_mode;

# Comments for current launch
our $test_comments;

# Test names -> test objects.
our %tests;

# Directory for results and report.
our $results_dir;

# A command to run autotest.
our $autotest_cmd;

#==== Helper functions ==================================================

# Prints any referenced structure (for debug purposes)
sub print_ref {
    my $href = shift;
    my $marg = shift;
    
    unless ( defined($marg) ) { $marg = 0; }

    if ( ref($href) eq "" ) {
        if ( ! defined ($href) ) { 
            print 'undef';
        } else {
            print '"'.$href.'"';
        }
    }
    elsif ( ref($href) eq "SCALAR" ) {
        print '"'.$$href.'"';
    }
    elsif ( ref($href) eq "HASH" ) {
        print '%{ '."\n";
        foreach my $key ( keys %$href ) {
            print " " x ($marg+3);
            print_ref ( $key, $marg+3);
            print ' => ';
            print_ref ( $href->{$key}, $marg+3 );
            print "\n";
        }
        print " " x $marg;
        print '  }%';
    }
    elsif ( ref($href) eq "ARRAY" ) {
        print '@[ '."\n";
        foreach my $elem ( @$href ) {
            print " " x ($marg+3);
            print_ref ( $elem, $marg+3);
            print "\n";
        }
        print " " x $marg;
        print ' ]@';
    }
    else {
        print 'Unknown type: "'.ref($href).'"';
    }
    if ($marg == 0) { print "\n"; }
}
#----------------------------------------------------------------------------

# Read and return file contents
sub read_file {
    my $filename = shift;
    my $text = "";
    
    open FILE, $filename or die "Can't open $filename";
    
    while (<FILE>) {
        $text .= $_;
    }
    close FILE;
    
    return $text;
}
#----------------------------------------------------------------------------

# Prints an error message to the console.
sub complain
{
    my $message = shift;
    $message = "error: $message\n";
    print STDERR $message;
}

#----------------------------------------------------------------------------

# Send a mail via *sendmail* utility.
# Parameter is a link to hash which shall contain field "TEXT" and may contain
# fields "TO", "FROM", "SUBJECT", "ATTACHMENT".
sub sendmail {
    my $params = shift;
    
    # Sendmail parameters are: 
    # -t   read the message to obtain recipients from the To: header
    # -oi  ignore dots alone on lines by themselves in incoming messages
    my $sendmail_cmd = "/usr/sbin/sendmail -t -oi";
    my $boundary = "Simple-Boundary--";
    debug_inform($sendmail_cmd);
    
    # Senmail reads its standard input
    #open(SENDMAIL, "| $sendmail_cmd") or die "Cannot find sendmail: $!";
    open(SENDMAIL, ">temp_mail") or die "Cannot find sendmail: $!";
    
    if ( $params->{SUBJECT} ) {
        $params->{SUBJECT} =~ s/[\n\r]+/ /g;
        print SENDMAIL "Subject: ".$params->{SUBJECT}."\n";
    }
    if ( $params->{TO} ) {
        print SENDMAIL "To: ".$params->{TO}."\n";
    }
    if ( $params->{FROM} ) {
        print SENDMAIL "From: ".$params->{FROM}."\n";
    }

    if ( $params->{ATTACHMENT} ) {
        print SENDMAIL "Content-Type: multipart/mixed; boundary=\"$boundary\"\n";
        print SENDMAIL "\nThis is a multi-part message in MIME format.\n";
        print SENDMAIL "--$boundary\n"
    }
    
    print SENDMAIL "Content-type: text/plain\n";
    print SENDMAIL "\n".$params->{TEXT}."\n";

    if ( $params->{ATTACHMENT} ) {
        print SENDMAIL "--$boundary\n";
        print SENDMAIL "Content-Type: application/octet-stream; name=\"".extract_filename($params->{ATTACHMENT})."\"\n"; 
        print SENDMAIL "Content-Transfer-Encoding: base64\n";
        print SENDMAIL "Content-Disposition: attachment; filename=\"".extract_filename($params->{ATTACHMENT})."\"\n";
        print SENDMAIL "\n";
        
        my $buf;
        open( FILE, $params->{ATTACHMENT} ) or die "$!";
        while (read(FILE, $buf, 60*57)) { print SENDMAIL encode_base64($buf); }
        close FILE;
    }
    close(SENDMAIL);
}

#----------------------------------------------------------------------------

# Prints a specified message and exits the program immediately
# via terminate ().
sub fail_nomail
{
    my $message = shift;

    # Print the error message to the console.
    complain($message);

    # Finish with an error code 2 (internal error).
    exit (2);
}

#----------------------------------------------------------------------------

# Like fail() but also sends e-mail if enabled.
sub fail
{
    my $message = shift;

    # Send an e-mail if the address is specified.
    if ($send_email) {
        &sendmail ( { 
            TO      => $tester_email,
            SUBJECT => "LSB autotest failed at $host_name",
            TEXT    => "The autotest run has failed with the following error:\n$message",
                    } );
    }

    # Fail normally.
    fail_nomail ($message);
}

#----------------------------------------------------------------------------

# Prints an informational message.
sub inform
{
    my $message = shift;

    # Get the current time.
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) = localtime ();

    if($verbose_lvl > 0)
    {
        # Print the message.
        #$message = sprintf ("[%02d:%02d:%02d] %s\n", $hour, $min, $sec, $message);
        $message = sprintf ("[%02d:%02d] %s\n", $hour, $min, $message);
        print ($message);
    }
}
#----------------------------------------------------------------------------

# Prints the message if verbose mode is enabled.
sub debug_inform {
    my $message = shift;
    if ($verbose_lvl >= 2) {
        inform ($message);
    }
}
#----------------------------------------------------------------------------

# Extracts filename from a full name
sub extract_filename
{
    my $fullname = shift;
    
    $fullname =~ m/([^\\\/]+)$/;
    return $1;
}


#----------------------------------------------------------------------------

# Runs a command using system() without termination of the process on failure
sub cmd
{
    my $command = shift;
    
    if ($verbose_lvl <2)
    {
        # Hide the output
        $command = "\{ $command; \} >/dev/null 2>&1";
    }
    
    debug_inform("% ".$command);
    
    # Run the command.
    my $result = system ($command);
    
    if( $result != 0 ) {
        # Warning on fail
        debug_inform ("Warning: code ".($result>>8)."(".($result&127).")"." has been returned.");
    }
    
    return $result;
}
#----------------------------------------------------------------------------

# Runs a command, terminating execution via fail() if the command fails.
sub run_command
{
    my $command = shift;

    if ($verbose_lvl < 1)
    {
        # Hide the output
        $command = "\{ $command \} >/dev/null 2>&1";
    }
    
    debug_inform("% ".$command);
    
    my $result = system ($command);

    # Check if it succeeded.
    if ($result != 0) {
        fail("Failed to execute: $command\nCode ".($result>>8)."(".($result&127).")"." has been returned.");
    }
}

#----------------------------------------------------------------------------

sub capture{
    my $command = shift;
    
    # Run the command.
    debug_inform("% ".$command);
    
    my $result = `$command`;
    
    if( $? != 0 ) {
        # Warning on fail
        debug_inform ("Warning: code ".($?>>8)."(".($?&127).")"." has been returned.");
    }
    
    return $result;
}

#----------------------------------------------------------------------------

# Runs a command that represents a specific test.
sub test_command
{
    my $journal_file = shift;
    my $title = shift;
    my $command = shift;
    my $test = shift;

    # Print the title (both to the journal and to the console).
    my $report = "$title: ";

    # Temporary files to hold the outputs of the command.
    my $stdout_file = "/tmp/lsb-autotest-stdout.txt";
    my $stderr_file = "/tmp/lsb-autotest-stderr.txt";

    # Run the command with catching standard and error output in files.
    my $result = system ("$command >$stdout_file 2>$stderr_file");

    # Read the standard and standard error output and reformat them
    # for HTML.
    my $stdout_text = `cat $stdout_file`;
    $stdout_text = "$stdout_text";
    my $stderr_text = `cat $stderr_file`;
    $stderr_text = "$stderr_text";

    if ($result != 0) {
        # If the result value is nonzero, the command has failed.
        $report .= "failed: exit value was $result, should be 0\n";
        $report .= "Standard output was: $stdout_text\n";
        $report .= "Error output was: $stderr_text";
    }
    elsif (-s $stderr_file) {
        # If the error output is not empty, it is also a failure.
        $report .= "failed: error output is not empty\n";
        $report .= "Standard output was: $stdout_text\n";
        $report .= "Error output was: $stderr_text";
    }

    else {
        # Try running the test command.
        if ($test) {
            $result = system ("\{ $test \} 2>&1 >/dev/null");
            if ($result != 0) {

                # If the result is nonzero, the test failed.
                $report .= "failed: test command failed with result $result: $test";
            }
            else {

                # Looks good.
                $report .= "OK";
            }
        }
        else {

            # Nothing to test, assume everything is allright.
            $report .= "OK";
        }
    }
    
    # Remove the temporary files.
    unlink($stdout_file);
    unlink($stderr_file);

    # Print the report to the journal.
    $journal_file->print("$report\n");
}

#----------------------------------------------------------------------------

# Writes the specified string to the specified file, removing original
# contents of the file (if any).
#
# Arguments:
# $1 - the file name.
# $2 - the string content to be written.
sub write_string_as_file
{
    my $file_name = shift;
    my $content = shift;

    # Open the file for writing.
    my $file;
    unless (open($file, "> $file_name")) {
        fail("Could not open file '$file_name' for writing");
    }

    # Print the string and close the file.
    print $file $content;
    close($file);
}

#----------------------------------------------------------------------------

# Creates the specified directory, or removes and re-creates it
# if it already exists.
sub create_empty_directory
{
    my $dir_name = shift;

    # If the directory already exists, remove it first.
    if (-d $dir_name) {

        # The directory already exists, delete it with all contents
        # and re-create to ensure it is empty and with standard access rights.
        run_command ("rm -rf $dir_name");
    }

    # Create the directory.
    run_command ("mkdir -p $dir_name");
}

#----------------------------------------------------------------------------

# Checks whether the specified tool is known to the shell.
sub does_shell_know
{
    my $tool_name = shift;
    return (system ("which $tool_name >/dev/null 2>/dev/null") == 0);
}

#----------------------------------------------------------------------------

# Checks whether the specified tool is available, terminating immediately
# if it is not.
sub check_for_tool
{
    my $tool_name = shift;
    
    if($tool_name eq "jdk_1.4.x")
    {
        unless (does_shell_know ("java")) {
            fail ("Could not find a required tool: $tool_name");
        }
        
        # Check the version number
        my $java_version = `java -version 2>&1`;
        unless ($java_version =~ m/^java version \"1\.(\d+)\.\w+\"/)
        {
            fail ("Could not realize java version.\n".
                "Olver requires SUN jdk_1.4.x or later to be ".
                "installed on your system.");
        }
        
        if( $1 < 4 )
        {
            fail ("Too old (or may be too new?) jdk version.\n".
                "Olver requires SUN jdk_1.4.x or later to be ".
                "installed on your system.");
        }
    }
    else
    {
        # Look if the shell knows this tool.
        unless (does_shell_know ("$tool_name")) {
            fail ("Could not find a required tool: $tool_name");
        }
    }
}

#----------------------------------------------------------------------------

sub backup_and_replace {
    my $old_file = shift;
    my $new_file = shift;
    
    unless (-f "$old_file.orig" ) {
        cmd ("cp -f $old_file $old_file.orig"); 
    }
    if (-f "$old_file.orig" ) {
        my $replace_cmd = "cp -f $new_file $old_file";
        my $result = cmd($replace_cmd);
        if ($result != 0) {
            inform ("Failed to copy: $replace_cmd");
            return 1;
        }
    } else {
        inform ("Failed to copy: cp -f $old_file $old_file.orig");
        return 1;
    }
    return 0;
}

1; #return value