# LSB Test Execution Framework
# Common test module
#
# Copyright (C) 2006 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2006 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# Common interface class for all test objects.
    
package Test_common;
use strict;

use Common;

=head ## Class description

class Test_common{
    NAME read:name;
    PACKAGES read:packages;
    SOURCES read:sources;
    BETA_SOURCES read:beta_sources;
    FILES read:files;
    START_TIME;
    FINISH_TIME;
    REPORT read:report, write:report($text);
    JOURNAL_TITLE;
    JOURNAL_ANCHOR;
    FAILURES_REPORTED;
    WARNINGS_REPORTED;
    FIPS_REPORTED;
    NOTICES_REPORTED;
    TESTS_TOTAL;
    TESTS_PASSED;
    TESTS_SKIPPED;
    TESTS_QUELLED;
    
    &check;
    &run;
    &cleanup;
    &is_available;
    &journal;
    &report;
}

=cut ##

# Class constructor
sub new {
    my $self  = {};
    $self->{NAME} = "";
    $self->{PACKAGES} = "";
    $self->{SOURCES} = "";
    $self->{BETA_SOURCES} = "";
    $self->{FILES} = "";
    $self->{START_TIME} = 0;
    $self->{FINISH_TIME} = 0;
    $self->{REPORTS} = {};
    $self->{JOURNAL_TITLE}  = "";
    $self->{JOURNAL_ANCHOR} = {};
    $self->{FAILURES_REPORTED} = {};
    $self->{WARNINGS_REPORTED} = {};
    $self->{FIPS_REPORTED} = {};
    $self->{NOTICES_REPORTED} = {};
    $self->{TESTS_TOTAL} = {};
    $self->{TESTS_PASSED} = {};
    $self->{TESTS_SKIPPED} = {};
    $self->{TESTS_QUELLED} = {};

    bless($self);
    return $self;
}

# Read methods for class fields
sub name { return shift->{NAME} };
sub packages { return shift->{PACKAGES} };
sub sources { return shift->{SOURCES} };
sub beta_sources { return shift->{BETA_SOURCES} };
sub files { return shift->{FILES} };

# Abstract class methods.
# Each test shall override necessary methods

# Do some initial environment checking.
sub check {
    my $self = shift;
}

# Run the test
sub run
{
    my $self = shift;
    
    my $command = "$autotest_cmd ".$self->name;
    debug_inform("% ".$command);
    my $result = system ($command) >> 8;
    
    return $result;
}

# Clean up. This method will be called even through the test fails
sub cleanup {
    my $self = shift;
}

# Check the test is available
sub is_available {
    my $self = shift;
    return 1;
}

# Store report files and create the record in journal
sub journal {
    my $self = shift;
}

sub open_report {
    my $self = shift;
    my $title = shift;
    my $anchor = shift;
    
    $self->{JOURNAL_TITLE} = $title;
    
    $self->{JOURNAL_ANCHOR}->{$title} = $anchor;
    
    $self->{FAILURES_REPORTED}->{$title} = 0;
    $self->{WARNINGS_REPORTED}->{$title} = 0;
    $self->{FIPS_REPORTED}->{$title} = 0;
    $self->{NOTICES_REPORTED}->{$title} = 0;
    $self->{TESTS_TOTAL}->{$title} = 0;
    $self->{TESTS_PASSED}->{$title} = 0;
    $self->{TESTS_SKIPPED}->{$title} = 0;
    $self->{TESTS_QUELLED}->{$title} = 0;
}

sub report {
    my $self = shift;
    my $text = shift;
    
    if ( $text ) {
        my $title = $self->{JOURNAL_TITLE};
        $self->{REPORTS}->{$title} .= $text;
    }
    else {
        return $self->{REPORTS};
    }
}

1; # return value