# LSB Test Execution Framework
# Common X11 test module
#
# Copyright (C) 2006 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2006 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# Common interface class for X11-oriented tests
    
package X11_common;
use strict;

use Tests::Test_common;
@X11_common::ISA = qw(Test_common);

use Common;

=head ##
Class X11_common Inherits Test_common {
    overrides &check;
}
=cut ##

sub new {
  my $self  = Test_common->new();
  bless($self);
  return $self;
}
sub check {
    my $self = shift;

    # Check for additional tools and packages.

    # The X virtual framebuffer server is used for testing.
    unless (does_shell_know ("Xvfb")) {
        inform ("Please install the Xvfb server first.");
    }
    
    # Fonts location for Ubunta
    
    unless (-e "/usr/X11R6/lib/X11/fonts") {
        cmd("cd /usr/X11R6/lib/X11; ln -s -f /usr/lib/X11/fonts");
    }
    
    # 100dpi fonts are needed for the X-oriented tests.
    unless (-f "/usr/X11R6/lib/X11/fonts/100dpi/timR24.pcf.gz") {
        inform ("Could not find 100dpi X fonts which are required "
            ."for X tests;\n(/usr/X11R6/lib/X11/fonts/100dpi/timR24.pcf.gz)\n"
            ."please install them first." );
    }
    
    Test_common::check($self);
}


sub run {
    my $self = shift;
    
    cmd ("killall -q Xvfb");
    cmd ("rm -f /tmp/.X?-lock");
    
    return Test_common::run($self);
}


sub cleanup {
    my $self = shift;
    
    cmd ("killall -q Xvfb");
    cmd ("rm -f /tmp/.X?-lock");
    
    return Test_common::cleanup($self);
}

1; # return value