/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#include "command_line.h"
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

/*-------------------------------------------------------------------------*/

void process_command_line(
    int argc,
    char ** argv,
    command_line_arg_callback_t arg_callback,
    command_line_finish_callback_t finish_callback,
    void * callback_user_data
)
{
    int arg_index;
    const char * arg;
    const char * next_arg;
    int callback_result;

    /* Scan all arguments (except the 0th one which is the program name). */
    arg_index = 1;
    while (arg_index < argc) {

        /* Read the argument and the following one (if available). */
        arg = argv[arg_index];
        next_arg = (arg_index + 1 < argc) ? argv[arg_index + 1] : NULL;

        /* Call the argument callback. */
        callback_result = arg_callback(
            arg,
            next_arg,
            callback_user_data
        );

        /* Stop processing if the callback returns 0. */
        if (callback_result == 0) {
            break;
        }

        /* The value returned from the callback specifies
         * how many arguments were processed (1 or 2).
         */
        assert(callback_result == 1 || callback_result == 2);
        arg_index += callback_result;
    }

    /* Call the finish callback if defined. */
    if (finish_callback) {
        finish_callback(callback_user_data);
    }
}

/*-------------------------------------------------------------------------*/
