/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef COMMAND_LINE_H
#define COMMAND_LINE_H

/*-------------------------------------------------------------------------*/

/**
 * Callback function called by process_command_line() for every argument
 * passed on the command line. A pointer to the argument is passed,
 * and a pointer to the next argument (if any) for the case that
 * this argument needs and additional option.
 *
 * The callback should return either the number of arguments processed
 * (1 if the next argument was not used, 2 if it was used),
 * or 0 to stop the command line processing.
 */
typedef int (*command_line_arg_callback_t)(
    const char * arg,
    const char * next_arg,
    void * user_data
);

/**
 * This callback is called by process_command_line() after
 * all command-line arguments were processed.
 */
typedef void (*command_line_finish_callback_t)(void * user_data);

/*-------------------------------------------------------------------------*/

/**
 * Processes a program command line, calling the specified callback
 * function for every command line argument.
 */
void process_command_line(
    int argc,
    char ** argv,
    command_line_arg_callback_t arg_callback,
    command_line_finish_callback_t finish_callback,
    void * callback_user_data
);

/*-------------------------------------------------------------------------*/

#endif
