/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#include "host_info.h"
#include "inform.h"
#include "string_utils.h"
#include "run_command.h"
#include "useful_macros.h"
#include <sys/utsname.h>

/*-------------------------------------------------------------------------*/

char * hostname = NULL;
char * host_os_name = NULL;
char * host_os_release = NULL;
char * host_os_version = NULL;
char * host_platform = NULL;
char * host_distro_name = NULL;
char * host_distro_release = NULL;
char * host_machine_triplet = NULL;

/*-------------------------------------------------------------------------*/

void read_host_info()
{
    struct utsname os_info;

    /* Determine the basic info about the host operating system. */
    EXPECT_ZERO(uname(&os_info), "Failed to call uname()");
    hostname = string_duplicate(os_info.nodename);
    host_os_name = string_duplicate(os_info.sysname);
    host_os_release = string_duplicate(os_info.release);
    host_os_version = string_duplicate(os_info.version);
    host_platform = string_duplicate(os_info.machine);

    /* Determine the vendor info using 'lsb_release'. */
    host_distro_name = string_duplicate(capture("lsb_release -s -i"));
    host_distro_release = string_duplicate(capture("lsb_release -s -r"));

    /* Build the machine description triplet. */
    host_machine_triplet = format_malloc("%s-linux-gnu", host_platform);
}

/*-------------------------------------------------------------------------*/

void print_host_info()
{
    /* Print the discovered information to console. */
    inform("Host name:            %s", hostname);
    inform("Host OS:              %s %s %s", host_os_name, host_os_release, host_os_version);
    inform("Host platform:        %s", host_platform);
    inform("Host distro:          %s %s", host_distro_name, host_distro_release);
    inform("Host machine triplet: %s", host_machine_triplet);
}

/*-------------------------------------------------------------------------*/
