/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef HTML_OUTPUT_H
#define HTML_OUTPUT_H

#include "string_utils.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

/*-------------------------------------------------------------------------*/

/**
 * This structure holds information for a HTML file being written.
 */
typedef struct {

    /**
     * The file pointer.
     */
    FILE * f;

} html_file;

/*-------------------------------------------------------------------------*/

/**
 * Opens a HTML file.
 */
html_file * html_open(const char * file_name);

/**
 * Closes the HTML file.
 */
void html_close(html_file * f);

/**
 * Prints a formatted text to the HTML file and adds a newline.
 * The text is not modified in any way; if it contains HTML tags,
 * they will be written to the file unchanged.
 */
void html_println(html_file * f, const char * format, ...);

/**
 * Prints a HTML head of specified level (1 is the topmost title).
 */
void html_print_head(html_file * file, unsigned int level, const char * text);

/**
 * Starts a paragraph.
 */
void html_start_paragraph(html_file * file);

/**
 * Finishes a paragraph.
 */
void html_end_paragraph(html_file * file);

/**
 * Starts a table.
 */
void html_start_table(html_file * f);

/**
 * Finishes a table.
 */
void html_end_table(html_file * f);

/**
 * Starts a table header.
 */
void html_start_table_header(html_file * f);

/**
 * Finishes the table header.
 */
void html_end_table_header(html_file * f);

/**
 * Prints a field of the table header.
 */
void html_print_table_header_field(html_file * f, const char * text);

/**
 * Starts a body of a table.
 */
void html_start_table_body(html_file * f);

/**
 * Finishes the table body.
 */
void html_end_table_body(html_file * f);

/**
 * Starts a table line.
 */
void html_start_table_line(html_file * f);

/**
 * Finishes a table line.
 */
void html_end_table_line(html_file * f);

/**
 * Finishes a table line and starts another one.
 */
void html_next_table_line(html_file * f);

/**
 * Prints a single field of a table.
 */
void html_print_table_field(html_file * f, const char * text);

/**
 * Prints a field that spans multiple columns.
 */
void html_print_multicolumn_field(html_file * f, const char * text,
    unsigned int columns);

/*-------------------------------------------------------------------------*/

#endif
