/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef INFORM_H
#define INFORM_H

#include <stdio.h>

/*-------------------------------------------------------------------------*/

/* Size of the buffer for the formatted message printed by fail(). */
#define FAIL_MESSAGE_BUFFER_SIZE 2048

/*-------------------------------------------------------------------------*/

/**
 * Callback function called by fail(). Receives a formatted message
 * that was previously printed to the console.
 */
typedef void (* fail_callback_t)(const char * msg);

/*-------------------------------------------------------------------------*/

extern int exit_code;

/*-------------------------------------------------------------------------*/

/**
 * Prints an informative message in printf-style (automatically adds
 * a newline).
 */
void inform(const char * message, ...);

/**
 * Prints a warning message in printf-style (automatically adds
 * a newline).
 */
void warn(const char * message, ...);

/**
 * Prints an error message in printf-style (automatically adds a newline)
 * and terminates the program with exit code of 127 (this should be
 * sufficiently different from any normal exit code).
 * If the tester e-mail is specified, the error is also sent to that address
 * by e-mail.
 * Uses a buffer with size of FAIL_MESSAGE_BUFFER_SIZE for formatting
 * the message; a longer message is truncated.
 */
void fail(const char * message, ...);

/**
 * Sets up a callback to be used by fail().
 */
void set_fail_callback(fail_callback_t callback);

/*-------------------------------------------------------------------------*/

#endif
