/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#include "output_file.h"
#include "lsb-autotest.h"
#include "time_utils.h"
#include "command_line.h"
#include "run_command.h"
#include "string_utils.h"
#include "host_info.h"
#include "file_utils.h"
#include "inform.h"
#include "sendmail.h"
#include <assert.h>
#include <stdlib.h>

/*-------------------------------------------------------------------------*/

void create_output_dir()
{
    assert(output_dir);

    /* Remove the previous output directory, if any. */
    remove_dir_recursive(output_dir);

    /* Create the output directory. */
    create_dir(output_dir);

    /* Create subdirectories. */
    create_dir("%s/autotest_info", output_dir);
    create_dir("%s/host_info", output_dir);
    create_dir("%s/test_journals", output_dir);
    create_dir("%s/test_logs", output_dir);
}

/*-------------------------------------------------------------------------*/

void create_output_file()
{
    inform("Writing the output archive");

    /* Compress the whole output directory into an archive. */
    run_command(
        "cd %s ; tar czvf %s * >/dev/null",
        output_dir,
        output_file
    );

    /* Send it by e-mail if requested. */
    if (tester_email) {
        /*
        run_command(
            "echo 'Please see the attached archive for the test results.'"
            " | mail -s 'LSB autotest results for %s' "
            "-a %s '%s'",
            hostname,
            output_file,
            tester_email
        );
        */
        //fail("TEST");
        char * topic_str = format_malloc("LSB autotest results for %s", hostname);
        sendmail ( 
                topic_str,
                tester_email,
                NULL,
                "Please see the attached archive for the test results.",
                output_file
            );
        free(topic_str);
    }

    inform("The output archive is ready: %s", output_file);
}

/*-------------------------------------------------------------------------*/
