/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef PACKAGE_H
#define PACKAGE_H

/*-------------------------------------------------------------------------*/

#define PACKAGE_MANAGER_UNKNOWN     0
#define PACKAGE_MANAGER_RPM         1
#define PACKAGE_MANAGER_DPKG        2

/*
 * Package manager
 */
extern int package_manager;

/*
 * How the wget tool should be called.
 */
extern const char * wget;

/*
 * This is the primary package source.
 */
extern const char * download_url;

/*
 * URLs where the packages reside in the primary source. They are determined
 * after the command line is analyzed and host platform detected.
 */
extern char * runtest_url;
extern char * appbat_url;
extern char * testdata_url;
extern char * beta_runtest_url;
extern char * beta_appbat_url;

/*
 * Platform identifiers and directory for packages. They are typically
 * the same as the host platform identifier, but not always.
 */
extern char * package_platform;
extern char * appbat_package_platform;
extern char * package_platform_dir;

/*-------------------------------------------------------------------------*/

/*
 * Builds URLs where to find the test packages according to
 * the host platform.
 */
void build_package_urls();

/*
 * Searches the package directory for a RPM file that has the given name
 * and matches the host platform, and returns a full path to the RPM file
 * (as a newly allocated string). Returns NULL if no suitable RPM is found.
 */
char * find_package_file(const char * name);

/*
 * Installs the specified package. Package repository is either "runtime"
 * for runtime test packages, or "appbat" for application battery packages.
 */
void install_package(const char * package_name, const char * package_repository);

/*
 * Returns true if the package is installed, false if it is not.
 */
int is_package_installed(const char * package_name);

/*
 * Checks whether a package is installed, and installs it if needed.
 */
void check_package(const char * package_name, const char * package_repository);

/*
 * Removes the package if it is installed (otherwise does nothing).
 */
void remove_package(const char * package_name);

/*
 * Removes all packages that are installed by the autotest suite.
 */
void purge_packages();

/*
 * Checks whether the specified data file is available,
 * and downloads it if needed.
 */
void check_data_file(const char * file_name);

/*
 * Identifies package manager ('rpm' or 'dpkg')
 */
void guess_package_manager();
/*-------------------------------------------------------------------------*/

#endif
