/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef RUN_COMMAND_H
#define RUN_COMMAND_H

/*-------------------------------------------------------------------------*/

/* Maximum size of buffer for formatting shell commands. */
#define COMMAND_BUFFER_SIZE 2048

/* Maximum size of the buffer for capturing output from a command. */
#define COMMAND_OUTPUT_BUFFER_SIZE 4096

/* Buffer used by capture() for catching output from commands. */
extern char command_output_buffer[];

/*-------------------------------------------------------------------------*/

/*
 * Runs the specified command. The command string can contain
 * printf-style substitution characters.
 * Terminates the program if the command fails (returns a nonzero
 * exit code or aborts).
 */
void run_command(const char * command, ...);

/*
 * Like run_command() but runs the command as the specified user.
 */
void run_command_as_user(const char * user_name, const char * command, ...);

/*
 * Runs the specified command. The command string can contain
 * printf-style substitution characters.
 * If the command terminates normally, although with nonzero exit code,
 * the exit code is returned. If the command terminates abnormally,
 * an error is printed and the program is terminated.
 */
int try_command(const char * command, ...);

/*
 * Runs the specified command and catches its output. The command string
 * can contain printf-style substitution characters.
 * The buffer used for the output is static and reused with each call
 * so the results must be copied to another buffer.
 */
char * capture(const char * command, ...);

/*
 * Checks whether the specified tool is known to the shell, terminating
 * the program immediately with an error message if the tool is not found.
 */
void check_for_tool(const char * name);

/*-------------------------------------------------------------------------*/

#endif
