#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sendmail.h"
#include "file_utils.h"
#include "string_utils.h"
#include "inform.h"
#include "base64.h"

/**
 * Send a mail via *sendmail* utility.
 * Parameter is a link to hash which shall contain field "TEXT" and may contain
 * fields "TO", "FROM", "SUBJECT", "ATTACHMENT".
 */
void sendmail ( 
        const char * subject,
        const char * to_addr,
        const char * from_addr,
        const char * text,
        const char * attachment
    )
{
    FILE * command_pipe;
    const char * wrong_attachment = NULL;
    
    /*
     * Sendmail parameters are: 
     * -t   read the message to obtain recipients from the To: header
     * -oi  ignore dots alone on lines by themselves in incoming messages
     */
    const char * sendmail_cmd = "/usr/sbin/sendmail -t -oi";
    const char * boundary = "Simple-Boundary--";
    char * buf = NULL;
        
    /* Senmail reads its standard input */
    command_pipe = popen(sendmail_cmd, "w");
    if (command_pipe == NULL) {
        fail("Could not start shell command: %s: %m", sendmail_cmd);
    }
    
    if ( subject ) {
        char * tmp = string_duplicate(subject);
        string_tr(tmp, '\n', ' ');
        string_tr(tmp, '\r', ' ');
        buf = format_malloc("Subject: %s\n", tmp);
        fwrite( buf, 1, strlen(buf), command_pipe);
        free(tmp);
        free(buf);
    }

    if ( to_addr ) {
        buf = format_malloc("To: %s\n", to_addr);
        fwrite( buf, 1, strlen(buf), command_pipe);
        free(buf);
    }

    if ( from_addr ) {
        buf = format_malloc("From: %s\n", from_addr);
        fwrite( buf, 1, strlen(buf), command_pipe);
        free(buf);
    }

    if ( attachment ) {
        if ( !file_exists(attachment) )
        {
            wrong_attachment = attachment;
            attachment = NULL;
        }
    }

    if ( attachment ) {
        buf = format_malloc(
                "Content-Type: multipart/mixed; boundary=\"%s\"\n"
                "\nThis is a multi-part message in MIME format.\n"
                "--%s\n", boundary, boundary 
            );
        fwrite( buf, 1, strlen(buf), command_pipe);
        free(buf);
    }
    
    buf = "Content-type: text/plain\n\n";
    fwrite( buf, 1, strlen(buf), command_pipe);
    
    if ( text )
    {
        fwrite( text, 1, strlen(text), command_pipe);
        
        if (wrong_attachment) 
        {
            buf = format_malloc( "\nFailed to attach a file: '%s'\n", wrong_attachment);
            fwrite( buf, 1, strlen(buf), command_pipe);
            free(buf);
        }
    }
        
    fwrite( "\n", 1, 1, command_pipe);

    if ( attachment ) {
        const char * filename = strrchr(attachment, '/')+1;
        
        buf = format_malloc(
                "--%s\n"
                "Content-Type: application/octet-stream; name=\"%s\"\n"
                "Content-Transfer-Encoding: base64\n"
                "Content-Disposition: attachment; filename=\"%s\"\n\n",
                boundary,
                filename,
                filename
            );
        fwrite( buf, 1, strlen(buf), command_pipe);
        free(buf);
        
        FILE* att_file = fopen( attachment, "r" );
        if ( att_file != NULL ) 
        {
            encode(att_file, command_pipe, 80 );
            fclose(att_file);
        }
    }
    
    fclose(command_pipe);
}
