/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef SUBSHELL_H
#define SUBSHELL_H

#include <utmp.h>
#include <stdio.h>

/*-------------------------------------------------------------------------*/

/*
 * PID of the subshell used for running tests.
 */
extern pid_t subshell_pid;

/*
 * File descriptor of the master side of the pseudo-terminal used for
 * communication with the subshell.
 */
extern int pty_master_fd;

/*
 * Stream attached to the master side of the pseudo-terminal.
 */
extern FILE * pty_master_file;

/*
 * An optional log file to capture the output of the subshell.
 */
extern FILE * subshell_log_file;

/*-------------------------------------------------------------------------*/

/*
 * Initializes the utmp entry structure to represent a login or logout on
 * the given terminal, with the given user name, shell PID,
 * and the current time as the login/logout time.
 * Note: For a logout entry, the user name is irrelevant and can be NULL.
 */
void init_utmp_entry(
    struct utmp * utmp_entry,
    const char * user_name,
    const char * tty_name,
    pid_t shell_pid,
    int login
);

/*
 * Writes an utmp entry to the utmp file, simulating login or logout.
 */
void write_utmp_entry(struct utmp * utmp_entry);

/*
 * Opens a subshell as the specified user in a separate terminal.
 */
void open_subshell(
    const char * user_name,
    const char * start_dir,
    const char * log_file_name
);

/*
 * Closes the previously opened subshell.
 */
void close_subshell();

/*
 * Sends the specified message to the subshell via the pseudo-terminal.
 * A newline is appended automatically.
 */
void subshell_send(const char * message);

/*
 * Waits until the specified text pattern arrives from the subshell.
 */
void subshell_expect(const char * pattern);

/*
 * Waits for the specified text pattern and then sends the message.
 */
void subshell_expect_and_reply(const char * pattern, const char * message);

/*
 * Waits until the subshell finishes.
 */
void wait_for_subshell();

/*-------------------------------------------------------------------------*/

#endif
