/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef TESTS_H
#define TESTS_H

/*-------------------------------------------------------------------------*/

/* Bits marking tests to be run. */
#define RUNTIME_TEST_BIT        0x00000001
#define LIBSTDCPP_TEST_BIT      0x00000002
#define CMDCHK_TEST_BIT         0x00000004
#define LIBCHK_TEST_BIT         0x00000008
#define VSW4_TEST_BIT           0x00000010
#define XTS5_TEST_BIT           0x00000020
#define DESKTOP_TEST_BIT        0x00000040
#define TCL_TEST_BIT            0x00000080
#define EXPECT_TEST_BIT         0x00000100
#define PYTHON_TEST_BIT         0x00000200
#define ALL_TESTS_MASK          0xffffffff

/* A bitmask that shows which tests should be run. */
extern int tests_to_run;

/*-------------------------------------------------------------------------*/

/*
 * Prints the set of tests selected to be run on the console using inform().
 */
void print_selected_tests();

/*
 * Prepares and runs tests selected in the 'tests_to_run' bitmask.
 */
void run_selected_tests();

/*
 * Test preparation routines. Each one installs all required test packages,
 * loads data files, prepares testing directory and users, etc.
 */
void prepare_runtime_test();
void prepare_libstdcpp_test();
void prepare_cmdchk_test();
void prepare_libchk_test();
void prepare_vsw4_test();
void prepare_xts5_test();
void prepare_desktop_test();
void prepare_tcl_test();
void prepare_expect_test();
void prepare_python_test();

/*
 * Test routines. Every test routine is executed in a separate process
 * so it can call fail() when something goes wrong, and the rest of the tests
 * can still be done.
 */
void run_runtime_test();
void run_libstdcpp_test();
void run_cmdchk_test();
void run_libchk_test();
void run_vsw4_test();
void run_xts5_test();
void run_desktop_test();
void run_tcl_test();
void run_expect_test();
void run_python_test();

/*-------------------------------------------------------------------------*/

#endif
