/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef TET_JOURNAL_INDEXED_H
#define TET_JOURNAL_INDEXED_H

/*-------------------------------------------------------------------------*/

#include "tet_journal.h"

/*-------------------------------------------------------------------------*/

/**
 * Prints the results in the journal in a format that's easy to access
 * in random order. There are two result file: the results file, which
 * holds the test results, and an index file which contains indices
 * to the results file for each test case.
 */
void tet_journal_print_indexable_results(
    TetJournal * journal,
    FILE * results_file,
    FILE * index_file
);

/**
 * Writes the .res and .idx file for the specified journal.
 */
void tet_journal_to_res_idx(TetJournal * journal,
    const char * base_file_name);

/**
 * Finds a result of the specified test point in the indexable results.
 */
int tet_journal_read_result_from_indexable_file(
    const char * results_file_name,
    const char * index_file_name,
    const char * test_case_name,
    unsigned int test_point_number
);

/**
 * Reads a single line from the .idx file, returning its contents:
 * the test case name (as a newly allocated string),
 * offset to the start of test case results in the .res file,
 * and the number of results.
 * Returns nonzero on success, zero on failure.
 */
int idx_read_line(
    FILE * idx_file,
    char ** test_case_name,
    unsigned long * offset,
    unsigned long * result_count
);

/**
 * Finds a test case information in the .idx file
 * and returns the appropriate offset of the results in the .res file
 * and the number of results.
 * Returns nonzero on success, zero on failure.
 */
int idx_find_test_case(
    FILE * idx_file,
    const char * test_case_name,
    unsigned long * offset,
    unsigned long * result_count
);

/**
 * Reads a test point result from the results file
 * (assuming that it was previously seeked to the right offset).
 * Returns the result, or -1 on an error.
 */
int res_read_test_point_result(
    FILE * res_file
);

/**
 * Seeks to a test point result in the results file.
 * Returns nonzero on success, zero on error.
 */
int res_seek_to_test_point_result(
    FILE * res_file,
    unsigned long test_case_offset,
    unsigned int test_point_number
);

/*-------------------------------------------------------------------------*/

#endif
