/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#ifndef USEFUL_MACROS_H
#define USEFUL_MACROS_H

#include "inform.h"
#include "string_utils.h"

/*-------------------------------------------------------------------------*/

/*
 * Evaluates the expression and checks if the result is zero
 * (typical for successful return of system calls). If it is not,
 * the program is terminated with the given error message.
 */
#define EXPECT_ZERO(expr, message) { \
    if ((expr) != 0) { \
        fail("%s: %m", message); \
    } \
}

/*
 * Evaluates the expression and checks if the result is the given value.
 * If it is not, the program is terminated with the given error message.
 */
#define EXPECT_INT(expr, value, message) { \
    if ((expr) != (value)) { \
        fail("%s: %m", message); \
    } \
}

/*
 * Evaluates the expression and checks if the result is NULL.
 * If it is not, the program is terminated with the given error message.
 */
#define EXPECT_NULL(expr, message) { \
    if ((expr) != NULL) { \
        fail("%s: %m", message); \
    } \
}

/**
 * Represents a 'for' loop that iterates the 'iterator' variable
 * through a linked list of objects which have a 'next' pointer.
 * The first object to be scanned is pointed to by 'start'.
 */
#define FOREACH(iterator, start) \
    for (iterator = start; iterator != NULL; iterator = iterator->next)

/*-------------------------------------------------------------------------*/

#endif
