/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#include "user_utils.h"
#include "inform.h"
#include "run_command.h"
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include <stdlib.h>
#include <string.h>

/*-------------------------------------------------------------------------*/

int user_exists(const char * user_name)
{
    struct passwd * user_info;

    /* Try getting the user info. */
    user_info = getpwnam(user_name);
    if (user_info == NULL) {
        return 0;
    }

    /* The user exists. */
    return 1;
}

/*-------------------------------------------------------------------------*/

int get_user_id(const char * user_name)
{
    struct passwd * user_info;

    /* Get the user info for the specified user name. */
    user_info = getpwnam(user_name);
    if (user_info == NULL) {
        fail("Could not find user '%s'", user_name);
    }

    /* Extract the user ID and return it. */
    return user_info->pw_uid;
}

/*-------------------------------------------------------------------------*/

char * get_user_home_dir(const char * user_name)
{
    struct passwd * user_info;

    /* Get the user info for the specified user name. */
    user_info = getpwnam(user_name);
    if (user_info == NULL) {
        fail("Could not find user '%s'", user_name);
    }

    /* Check if the user has a home directory. */
    if ((user_info->pw_dir == NULL) || (user_info->pw_dir[0] == '\0')) {
        fail("User '%s' has no home directory", user_name);
    }

    /* Return the home directory as a newly allocated string. */
    return strdup(user_info->pw_dir);
}

/*-------------------------------------------------------------------------*/

int get_user_primary_group_id(const char * user_name)
{
    struct passwd * user_info;

    /* Get the user info. */
    user_info = getpwnam(user_name);
    if (user_info == NULL) {
        fail("Could not find user '%s'", user_name);
    }

    /* Extract the group ID and return it. */
    return user_info->pw_gid;
}

/*-------------------------------------------------------------------------*/

int get_group_id(const char * group_name)
{
    struct group * group_info;

    /* Get info about the group. */
    group_info = getgrnam(group_name);
    if (group_info == NULL) {
        fail("Could not find group '%s'", group_name);
    }

    /* Extract the group ID and return it. */
    return group_info->gr_gid;
}

/*-------------------------------------------------------------------------*/

void add_user(const char * user_name)
{
    /* Return immediately if the user already exists. */
    if (user_exists(user_name)) {
        return;
    }

    /* Add the user. */
    run_command("useradd -m %s", user_name);
}

/*-------------------------------------------------------------------------*/
