#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2006 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Fcntl ':flock';

my $DOCUMENT_ROOT = $0;
$DOCUMENT_ROOT =~ s/\/[^\/]+$//;

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

eval(`cat $DOCUMENT_ROOT/Templates.pl`);

$PeerIP = "" if (!$PeerIP);
my $conf_page = ($_POST{"certify"} ? "tests_cert.pl" : "tests_conf.pl");
my $session_profile = ($_COOKIE{"session_id"} ? "~session.$PeerIP.".$_COOKIE{"session_id"} : "");

my $error = "";
my $new_url = "";
my $save_profile_name = "";

if (!-d "$DOCUMENT_ROOT/profiles") {
	if (!mkdir("$DOCUMENT_ROOT/profiles")) {
		$error = "Cannot create the profiles directory!<br />$!";
	}
}

if ($error eq "") {
	if (!$_POST{"action"} or ($_POST{"action"} eq "")) {
		$error = "Please, do not call this script directly!";
	}
	elsif ($_POST{"action"} eq "load_profile") {
		if (!$_POST{"load_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"load_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid file name!";
		}
		else {
			system ("cp \"$DOCUMENT_ROOT/profiles/$_POST{load_profile_name}\" \"$DOCUMENT_ROOT/profiles/$session_profile\" > /dev/null");
			$new_url = "$conf_page?profile=$_POST{load_profile_name}";
		}
	}
	elsif ($_POST{"action"} eq "refresh_list") {
		$new_url = "tests_conf_refresh.pl";
		if ($_POST{"certify"}) {
			$new_url .= "?cert=".$_POST{"certify"};
		}
		$save_profile_name = $session_profile;
	}
	elsif ($_POST{"action"} eq "run_tests") {
		my $tests_exec = 0;
		if (-f "/tmp/tests_output.8888") {
			if (open(FILE, "< /tmp/tests_output.8888")) {
				my $PID = <FILE>;
				$PID =~ s/PID: (\d+),.*/$1/g;
				if (is_process_finished($PID) and flock(FILE, LOCK_EX | LOCK_NB)) {
#					unlink("/tmp/tests_output.8888");
					rename("/tmp/tests_output.8888", "/tmp/tests_output.8888.old");
					flock(FILE, LOCK_UN);
					$tests_exec = 1;
				}
				close(FILE);
			}
		}
		else {
			$tests_exec = 1;
		}

		if (!$tests_exec) {
			$save_profile_name = $session_profile;
			$new_url = "$conf_page?error=1";
		}
		else {
			if ($session_profile eq "") {
				$error = "Cannot retrieve session ID! Please, allow cookies in your browser.";
			}
			else {
				$new_url = "tests_exec.pl?profile=$session_profile";
				if ($_POST{"certify"}) {
					$new_url .= "&cert=".$_POST{"certify"};
				}
				$save_profile_name = $session_profile;
			}
		}
	}
	elsif ($_POST{"action"} eq "save_profile") {
		if ($_POST{"save_profile_name"}) {
			$save_profile_name = $_POST{"save_profile_name"};
		}
		elsif ($_POST{"load_profile_name"}) {
			$save_profile_name = $_POST{"load_profile_name"};
		}
		if ($save_profile_name eq "") {
			$error = "The profile name is not specified!";
		}
		elsif ($save_profile_name =~ m/[\\\/]/) {
			$error = "Invalid file name!";
		}
		else {
			$new_url = "$conf_page?profile=$save_profile_name";
		}
	}
	elsif ($_POST{"action"} eq "delete_profile") {
		if ($_POST{"load_profile_name"}) {
			my $del_name = $_POST{"load_profile_name"};
			if ($del_name =~ m/[\\\/]/) {
				$error = "Invalid file name!";
			}
			else {
				unlink("$DOCUMENT_ROOT/profiles/$del_name");
				$save_profile_name = $session_profile;
				$new_url = $conf_page;
			}
		}
		else {
			$error = "The profile name is not specified!";
		}
	}
}

if ($error eq "") {
	if ($save_profile_name ne "") {
		my $full_save_name = "$DOCUMENT_ROOT/profiles/$save_profile_name";
		if (open(FILE, "> $full_save_name")) {
			my $architecture = ($_POST{"architecture"} or "ia32");
			my $lsb_version = ($_POST{"lsb_version"} or "3.1");
			print FILE "CERTIFY:         ".$_POST{"certify"}."\n"      if ($_POST{"certify"});
			print FILE "NAME:            ".$_POST{"tester_name"}."\n"  if ($_POST{"tester_name"});
			print FILE "ORGANIZATION:    ".$_POST{"organization"}."\n" if ($_POST{"organization"});
			if ($_POST{"tester_email"}) {
				print FILE "EMAIL:           ".$_POST{"tester_email"}."\n";
				print FILE "SEND_EMAIL:      ".($_POST{"send_email_report"} ? "1" : "0")."\n";
			}

			print FILE "VERBOSE_LEVEL:   ".($_POST{"verbose_level"} ? $_POST{"verbose_level"} : "1")."\n";
			print FILE "KEEP_PACKAGES:   ".($_POST{"keep_packages"} ? "1" : "0")."\n";
			print FILE "SHORT_SCENARIOS: ".($_POST{"short_scen"} ? "1" : "0")."\n";
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "USE_INTERNET:    ".($_POST{"use_internet"} ? "1" : "0")."\n";
			print FILE "LSB_VERSION:     $lsb_version\n";

			if ($_POST{"comments"}) {
				my @comments_lines = split(/[\r\n]+/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			foreach (sort grep /mod-$architecture-$lsb_version-/, keys %_POST) {
				my $tmp = $_;
				$tmp =~ s/^mod-/modstage-/;
				s/^mod-$architecture-$lsb_version-//;
				print FILE "$_ ".$_POST{$tmp}."\n";
			}

			close(FILE);

			if ($_POST{"action"} eq "save_profile") {
				system ("cp \"$full_save_name\" \"$DOCUMENT_ROOT/profiles/$session_profile\" > /dev/null");
			}
		}
		else {
			$error = "Cannot open file $save_profile_name for writing!<br />$!";
		}
	}
}

if ($error eq "") {
	print "HTTP/1.0 302 Moved\nLocation: $new_url\n\n";
}
else {
	print "HTTP/1.0 200 OK\n";
	print "Content-type: text/html\n\n";

	print_header("Error Occured", ($_POST{"certify"} ? "cert" : "conf"));

	print <<DATA;
<h1>Saving the profile failed!</h1>
<p><font color="red"><b>$error</b></font><br><br>
<a href="javascript:history.go(-1);">Back to previous page</a></p>
DATA

	print_footer();
}
