#!/usr/bin/perl -w

# LSB Test Execution Framework
# Kill Process Tree Module (tests_kill.pl)
#
# Copyright (C) 2006 Free Standards Group. All rights reserved.
#
# This program has been developed by ISP RAS for FSG.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

my $old_handle = select (STDOUT);
$| = 1;
select ($old_handle);

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

my %CHLD = ();
sub build_tree() {
	foreach (split(/[\r\n]+/, `ps -A -o pid= -o ppid=`)) {
		s/^\s+//g;
		my ($ps_pid, $ps_ppid) = split(/\s+/, $_, 2);
		if (!defined($CHLD{$ps_ppid})) {
			my @temp = ();
			$CHLD{$ps_ppid} = \@temp;
		}
		push @{$CHLD{$ps_ppid}}, $ps_pid;
	}
}

my %to_kill = ();
sub kill_tree($) {
	my $pid = $_[0];
	return if (!$pid or ($pid == getppid()));
	system("kill -SIGTERM ".$pid);
	$to_kill{$pid} = 1;
	my $temp = $CHLD{$pid};
	return if (!$temp);
	foreach my $i (@$temp) {
		&kill_tree($i);
	}
}

sub is_tree_killed() {
	for (my $i=0; $i<10; $i++) {
		foreach (keys %to_kill) {
			if (is_process_finished($_)) {
				delete $to_kill{$_};
			}
		}
		if (scalar(keys %to_kill) == 0) {
			return 1;
		}
		else {
			select(undef, undef, undef, 0.3);
		}
	}
	return 0;
}

my $DOCUMENT_ROOT = $0;
$DOCUMENT_ROOT =~ s/\/[^\/]+$//;

eval(`cat $DOCUMENT_ROOT/Templates.pl`);

if (!$_GET{"pid"}) {
	print "HTTP/1.0 200 OK\n";
	print "Content-type: text/html\n\n";

	print_header("Error Occured", "admin");

	print <<DATA;
<h1>PID is not specified!</h1>
<a href="javascript:history.go(-1);">Back to previous page</a></p>
DATA

	print_footer();
}
else {
	build_tree();
	kill_tree($_GET{"pid"});
	if (is_tree_killed()) {
		if ($_GET{server}) {
			print "HTTP/1.0 200 OK\n";
			print "Content-type: text/html\n\n";
			print "<html><body><h2>The server has been successfully stopped!</h2><p>Bye, see you later...</p></body></html>";
			exit;
		}
		else {
			if ($_GET{"exec"}) {
				my $profile = ($_GET{"profile"} ? "&profile=".$_GET{"profile"} : "");
				print "HTTP/1.0 302 Moved\nLocation: tests_exec.pl?killed=1$profile\n\n";
			}
			else {
				print "HTTP/1.0 302 Moved\nLocation: tests_admin.pl\n\n";
			}
		}
	}
	else {
		print "HTTP/1.0 200 OK\n";
		print "Content-type: text/html\n\n";

		print_header("Error Occured", "admin");

		print <<DATA;
<h1>Could not kill the process!</h1>
<ul>
  <li><a href="javascript:location.reload();">Retry</a></li>
  <li><a href="javascript:history.go(-1);">Back to previous page</a></li>
</ul>
DATA

		print_footer();
	}
}
