/*-------------------------------------------------------------------------*/
/* LSB Autotest, the automatic LSB test running tool                       */
/*                                                                         */
/* Copyright (c) 2006 SUSE LINUX Products GmbH                             */
/*                                                                         */
/* Author: Jiri Dluhos <jdluhos@suse.cz>                                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License version 2 as       */
/* published by the Free Software Foundation.                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the Free Software             */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              */
/* MA 02110-1301, USA.                                                     */
/*-------------------------------------------------------------------------*/

#include "backdoor.h"
#include "inform.h"
#include "run_command.h"
#include "useful_macros.h"
#include "file_utils.h"
#include <assert.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

/*-------------------------------------------------------------------------*/

/*
 * Makes a backup copy of the original /etc/pam.d/su config file.
 */
static void backup_su_config(void)
{
    if ( !file_exists ("/etc/pam.d/su.orig0") ) {
        run_command("cp /etc/pam.d/su /etc/pam.d/su.orig0");
    }
}

/*-------------------------------------------------------------------------*/
    
/*
 * Finds where the pam_wheel.so module is.
 */
static const char * find_pam_wheel_module(void)
{
    if (file_exists("/lib/security")) {
        return "/lib/security/$ISA/pam_wheel.so";
    }
    else if (file_exists("/lib64/security")) {
        return "/lib64/security/$ISA/pam_wheel.so";
    }
    else {
        fail("Could not find the pam_wheel.so module");
        return NULL;
    }
}
    
/*-------------------------------------------------------------------------*/
    
void install_backdoor()
{
    sstring * wheel_line = sstring_create();
    char * line_buffer = NULL;
    int buffer_size = 0;
    int line_number = 0;
    FILE * f_orig;
    FILE * f;

    inform("Installing the vsx0-to-root backdoor (will be removed at end)");

    backup_su_config();

    f_orig = open_file_for_reading("/etc/pam.d/su.orig0");
    f = open_file_for_writing("/etc/pam.d/su");

    sstring_printf(
        wheel_line,
        "auth       sufficient %s trust group=vsxg0 use_uid",
        find_pam_wheel_module()
    );

    for (;;) {

        line_number++;
        
        /* Read the next line from the file. */
        if ( !read_line(f_orig, &line_buffer, &buffer_size) ) {
            break;
        }
        assert(line_buffer);
        
        /* Insert the wheel_line at the beginning of the file after #%PAM-1.0 */
        if ( line_number == 1 ) {
            if ( string_starts_with(line_buffer, "#%") ) {
                fprintf(f, "%s\n", line_buffer);
                fprintf(f, "%s\n", CSTRING(wheel_line));
            } else {
                fprintf(f, "%s\n", CSTRING(wheel_line));
                fprintf(f, "%s\n", line_buffer);
            }
        } else {
            fprintf(f, "%s\n", line_buffer);
        }
    }
    
    fclose(f);
    fclose(f_orig);

    sstring_delete(wheel_line);
}

/*-------------------------------------------------------------------------*/

void remove_backdoor()
{
    if ( file_exists("/etc/pam.d/su.orig0") ) {

        inform("Removing the vsx0-to-root backdoor");

        /* Restore original PAM settings and remove the special user list. */
        EXPECT_ZERO(
            rename("/etc/pam.d/su.orig0", "/etc/pam.d/su"),
            "Could not restore original /etc/pam.d/su file"
        );
    }
}
